; Night and Day
;
; von  Krischan
; Demo inspiriert durch Tag/Nachtwechsel bei den Gothic-Spielen

AppTitle "Night and Day"
Graphics3D 1024,768,32,1

; Startwerte
winkel#=3300.0				; Position der Sonne
speed#=0.5					; Geschwindigkeit der Simulation
sdetail=8					; Anzahl Seiten der Lichtkugel
lod=2000					; Terrain LOD

; Pivot fr Steuerung und globale Variablen
Global cam, player
Global cam_pitch#, cam_yaw#
Global mvx#,mvy#,mvz#,targetpitch#,targetyaw#
player=CreatePivot()

; Verlaufsdatei laden
daylight=LoadImage("data\light.bmp")
daysize=ImageWidth(daylight)

; Farbfelder dimensionieren
Dim c1(daysize),c2(daysize),c3(daysize)
Dim s1(daysize),s2(daysize),s3(daysize)
Dim alpha#(daysize),xalpha#(daysize)

; Farbfelder auslesen
For x=0 To daysize-1

	rgb1=ReadPixel(x,0,ImageBuffer(daylight))
	rgb2=ReadPixel(x,18,ImageBuffer(daylight))
	rgb3=ReadPixel(x,36,ImageBuffer(daylight))
	rgb4=ReadPixel(x,54,ImageBuffer(daylight))
	
	c1(x)=(rgb1 And $FF0000)/$10000
	c2(x)=(rgb1 And $FF00)/$100
	c3(x)=(rgb1 And $FF)
	
	s1(x)=(rgb2 And $FF0000)/$10000
	s2(x)=(rgb2 And $FF00)/$100
	s3(x)=(rgb2 And $FF)
	
	a1=(rgb3 And $FF0000)/$10000
	a2=(rgb3 And $FF00)/$100
	a3=(rgb3 And $FF)

	x1=(rgb4 And $FF0000)/$10000
	x2=(rgb4 And $FF00)/$100
	x3=(rgb4 And $FF)
		
	alpha#(x)=Floor((a1+a2+a3)/3)/255
	xalpha#(x)=Floor((x1+x2+x3)/3)
			
Next

; Grafik fr sptere Ausgabe skalieren
ScaleImage(daylight,0.1,1)

; Kamera
cam=CreateCamera(player)
PositionEntity cam,0,50,0
CameraZoom cam,1
CameraRange cam,0.1,10000
CameraFogRange cam,0,250

; Kamera ungefhr auf Sonnenaufgangsposition ausrichten
cam_pitch=-25
cam_yaw=180
targetpitch=-25
targetyaw=180

; Skyboxes: Sterne, Himmel, Nebel
stars=MakeSkyBox("data\night_","000","000","000","000","top","png",1003,3)
sky=MakeSkyBox("data\sky_","000","090","180","270","top","jpg",1002,2)
fog=MakeSkyBox("data\fog_","000","090","180","270","top","jpg",1001,1)

; Pivotpunkt erzeugen (daran hngt die Himmelsmechanik)
pivot=CreatePivot()
TurnEntity pivot,180,0,0
PositionEntity pivot,0,0,0

; Landschaft
terrain=LoadTerrain("data\heightmap.jpg")
TerrainDetail terrain,lod,True
ScaleEntity terrain,2,80,2
grass_tex=LoadTexture("data\mountain.jpg")
EntityTexture terrain,grass_tex
ScaleTexture grass_tex,20,20
PositionEntity terrain,-500,-20,-500

; Sphre fr Lichtquelle und zweite Wolkenschicht
sphere=CreateSphere(sdetail)
PositionEntity sphere,0,-250,0
ScaleEntity sphere,1000,1000,1000
EntityColor sphere,128,128,128
EntityFX sphere,16
EntityBlend sphere,3
EntityAlpha sphere,0.5
RotateEntity sphere,0,0,90
skytex=LoadTexture("data\clouds.jpg")
EntityTexture sphere,skytex

; Sonne als Sprite
sun=LoadSprite("data\sun.bmp",2+256)
PositionEntity sun,0,0,1000
EntityParent sun,pivot
EntityFX sun,9

; Mond als Sprite
moon=LoadSprite("data\moon.bmp",2+256)
PositionEntity moon,0,0,-1000
EntityParent moon,pivot
EntityFX moon,9

; Sonnenschein als Sprite
sunshine=LoadSprite("data\sunshine.bmp",2+256)
PositionEntity sunshine,0,0,1000
EntityParent sunshine,pivot

; Sonnenlicht, welches die Sphre erhellt
sunlight=CreateLight(1)
PositionEntity sunlight,0,0,1500
LightColor sunlight,255,255,220
EntityParent sunlight,pivot
AmbientLight 64,64,64

; Zeitberechnungsteiler
hdiv#=daysize/24
mdiv#=hdiv#/60

; Mauszeiger in die Mitte
MoveMouse GraphicsWidth()/2,GraphicsHeight()/2

; Hauptschleife
While Not KeyHit(1)

	; Winkel um speed erhhen
	winkel#=winkel#+speed#
	
	; Tastatursteuerung
	If KeyDown(205) Then winkel#=winkel#+(speed#*5)		; Pfeil rauf = vorwrts
	If KeyDown(203) Then winkel#=winkel#-(speed#*5)		; Pfeil runter = rckwrts
	If KeyHit(57) Then si=1-si							; Space = Farbverlauf ein/ausblenden
	If KeyHit(59) Then fo=1-fo							; F1 = Nebel an/aus
		
	; Nebelfarbe
	CameraFogMode cam,fo
	CameraClsColor cam,c1(w),c2(w),c3(w)
	CameraFogColor cam,c1(w)/1.5,c2(w)/1.5,c3(w)/1.5

	; Nebel Alpha
	If fo=1 Then EntityAlpha fog,0.75 Else EntityAlpha fog,0
		
	; Winkel begrenzen
	If winkel#>3599 Then winkel#=0
	If winkel#<0 Then winkel#=3599
	w=Int(winkel#)
		
	; Skybox und Terrain beleuchten, Sonne frben, Himmel faden
	EntityColor sky,c1(w),c2(w),c3(w)
	EntityColor fog,c1(w)/2,c2(w)/2,c3(w)/2
	EntityColor terrain,c1(w),c2(w),c3(w)

	EntityColor sun,s1(w),s2(w),s3(w)
	EntityColor sunshine,s1(w),s2(w),s3(w)
	EntityColor moon,255,255,255
	
	EntityAlpha sky,alpha#(w)
	EntityAlpha sunshine,xalpha#(w)/384
	LightColor sunlight,xalpha#(w),xalpha#(w),xalpha#(w)
	
	; Wolken und Sterne etwas bewegen
	TurnEntity sphere,0,-speed#/10,0
	TurnEntity stars,0,-speed#/10,0
	
	; Sonne, Sonnenschein und Mond sind am Horizont grsser
	size#=(1-Sin(winkel#/10)/3)
	ScaleSprite sun,250*size#,250*size#
	ScaleSprite moon,200*size#,200*size#
	ScaleSprite sunshine,500*size#,500*size#
			
	; Sonnenbewegung anhand des Winkels, das Licht zeigt immer auf Sphere	
	RotateEntity pivot,(winkel#/10),0,0
	PointEntity sunlight,sphere
	
	; FPS counter
	FPS_C=FPS_C+1 
	If ms<MilliSecs() 
		ms=MilliSecs()+1000 
		FPS=FPS_C 
		FPS_C=0 
	EndIf 

	; Maussteuerung
	PlayerControls()
	
	; alles berechnen
	UpdateWorld
	RenderWorld
	
	; Uhrzeit aus Winkel berechnen
	hour=Floor(winkel#/hdiv#)
	minu=(winkel-(hour*hdiv#))/mdiv#
	hour=7+hour : If hour>23 Then hour=hour-24
	If(hour<10) Then stunde$="0"+Str(hour) Else stunde$=Str(hour)
	If(minu<10) Then minute$="0"+Str(minu) Else minute$=Str(minu)
	
	; Infos ausgeben
	Color 255,255,255
	Text 0,0,"Tris: "+TrisRendered()+" FPS: "+FPS
	Text 0,20,stunde$+":"+minute$+" Uhr"

	; Farbverlauf ausgeben
	If si Then
		DrawImage daylight,0,50
		Color 255,0,0
		Rect (w/10),50,1,72
		Color 255,255,255
		
		Text ImageWidth(daylight)+5,52,"Himmel"
		Text ImageWidth(daylight)+5,70,"Sonne"
		Text ImageWidth(daylight)+5,88,"Himmel Alpha"
		Text ImageWidth(daylight)+5,106,"Licht Alpha"
	EndIf
	
	; Bufferflip
	Flip 1

Wend

End

; Funktion um eine Skybox zu erstellen
Function MakeSkyBox( pre$,front$,Right$,back$,Left$,Top$,ext$,scale,order ) 

	m=CreateMesh() 

	;front face 
	b=LoadBrush( pre$+front$+"."+ext$,49) 
	s=CreateSurface( m,b ) 
	AddVertex s,-1,+1,-1,0,0:AddVertex s,+1,+1,-1,1,0 
	AddVertex s,+1,-1,-1,1,1:AddVertex s,-1,-1,-1,0,1 
	AddTriangle s,0,1,2:AddTriangle s,0,2,3 
	FreeBrush b 

	;right face 
	b=LoadBrush( pre$+Right$+"."+ext$,49 ) 
	s=CreateSurface( m,b ) 
	AddVertex s,+1,+1,-1,0,0:AddVertex s,+1,+1,+1,1,0 
	AddVertex s,+1,-1,+1,1,1:AddVertex s,+1,-1,-1,0,1 
	AddTriangle s,0,1,2:AddTriangle s,0,2,3 
	FreeBrush b 

	;back face 
	b=LoadBrush( pre$+back$+"."+ext$,49 ) 
	s=CreateSurface( m,b ) 
	AddVertex s,+1,+1,+1,0,0:AddVertex s,-1,+1,+1,1,0 
	AddVertex s,-1,-1,+1,1,1:AddVertex s,+1,-1,+1,0,1 
	AddTriangle s,0,1,2:AddTriangle s,0,2,3 
	FreeBrush b 

	;left face 
	b=LoadBrush( pre$+Left$+"."+ext$,49 ) 
	s=CreateSurface( m,b ) 
	AddVertex s,-1,+1,+1,0,0:AddVertex s,-1,+1,-1,1,0 
	AddVertex s,-1,-1,-1,1,1:AddVertex s,-1,-1,+1,0,1 
	AddTriangle s,0,1,2:AddTriangle s,0,2,3 
	FreeBrush b 

	;top face 
	b=LoadBrush( pre$+top$+"."+ext$,49 ) 
	s=CreateSurface( m,b ) 
	AddVertex s,-1,+1,+1,0,1:AddVertex s,+1,+1,+1,0,0 
	AddVertex s,+1,+1,-1,1,0:AddVertex s,-1,+1,-1,1,1 
	AddTriangle s,0,1,2:AddTriangle s,0,2,3 
	FreeBrush b 

	ScaleMesh m,scale,scale,scale
	FlipMesh m 
	EntityFX m,9
	EntityOrder m,order
	
	Return m 

End Function

; Spielersteuerung
Function PlayerControls()

	mxspd# = MouseXSpeed()*0.25
	myspd# = MouseYSpeed()*0.25

	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2

	targetpitch = targetpitch + myspd
	targetpitch = ClampValue(targetpitch, -85,85)
	targetyaw = targetyaw - mxspd	

	cam_pitch = cam_pitch + (targetpitch - cam_pitch)/8.0
	cam_yaw = cam_yaw + (targetyaw - cam_yaw)/8.0
	
	RotateEntity player,0,cam_yaw,0
	RotateEntity cam,cam_pitch,0,0

	If KeyDown(200) Then mvz=mvz+1
	If KeyDown(208) Then mvz=mvz-1
	
	If KeyDown(30) Then mvx=mvx-1
	If KeyDown(32) Then mvx=mvx+1
	If KeyDown(17) Then mvz=mvz+1
	If KeyDown(31) Then mvz=mvz-1
	
	TranslateEntity player,0,mvy,0
	MoveEntity player,mvx,0,mvz

	mvx=mvx/1.2
	mvy=mvy/1.2
	mvz=mvz/1.2
	
End Function

; vertikale Begrenzung
Function ClampValue(Original#, low#, high#)
	If Original<low  Then Return low
	If Original>high Then Return high
	Return Original
End Function